/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* global window, tizen, console */

/**
 * Application model module.
 * It's responsible for managing media sources
 * and managing media keys API.
 *
 * @module app.model
 * @namespace app.model
 * @memberof app
 */

// make sure that "app" namespace is created
window.app = window.app || {};

// strict mode wrapper function
(function defineAppModel(app) {
    'use strict';

    /**
     * Content filter object.
     *
     * @memberof app.model
     * @private
     * @type {AttributeFilter}
     */
    var filter = new tizen.AttributeFilter('type', 'EXACTLY', 'AUDIO'),

        /**
         * Stores all audio objects found by ContentManager.
         *
         * @memberof app.model
         * @private
         * @type {AudioContent[]}
         */
        contentList = [],

        /**
         * Stores content identifier for Music directory.
         *
         * @memberof app.model
         * @private
         * @type {string}
         */
        directoryId = '';

    /**
     * Returns array of audio objects.
     *
     * @memberof app.model
     * @public
     * @returns {AudioContent[]}
     */
    function getAudioContentList() {
        return contentList;
    }

    /**
     * Success callback for searching ContentManager.
     * Assigns found content to contentList variable.
     * Fires "model.ready" event if there are music files available,
     * "model.noaudiofound" otherwise.
     *
     * @memberof app.model
     * @private
     * @param {AudioContent[]} audioContent
     * @fires "model.ready"
     * @fires "model.noaudiofound"
     */
    function onGetAudioContent(audioContent) {
        contentList = audioContent;

        if (audioContent.length > 0) {
            app.common.dispatchEvent('model.ready');
        } else {
            app.common.dispatchEvent('model.noaudiofound');
        }
    }

    /**
     * Error callback for searching ContentManager.
     * Displays message on the console when error occurs
     * while searching ContentManager.
     *
     * @memberof app.model
     * @private
     * @param {Error} error
     */
    function onGetAudioContentError(error) {
        console.error('onGetAudioContent error: ', error);
    }

    /**
     * Searches audio content in Music directory.
     *
     * @memberof app.model
     * @private
     */
    function searchContent() {
        try {
            tizen.content.find(onGetAudioContent, onGetAudioContentError,
                directoryId, filter);
        } catch (error) {
            console.error('searchContent error: ', error);
        }
    }

    /**
     * Success callback for obtaining content directories.
     * Sets content identifier of Music directory.
     * Launches audio files searching operation if directory is found.
     * Fires "model.nostoragefound" event if no Music storage found.
     * Fires "model.noaudiofound" event if no audio found.
     *
     * @memberof app.model
     * @private
     * @param {ContentDirectory[]} directories
     * @fires "model.nostoragefound"
     * @fires "model.noaudiofound"
     */
    function onGetDirectoriesSuccess(directories) {
        var i = 0,
            dirLength = directories.length;

        if (dirLength) {
            for (i; i < dirLength; i += 1) {

                if (directories[i].title === 'Music') {
                    directoryId = directories[i].id;
                    searchContent();
                    break;
                }
            }

            if (!directoryId) {
                app.common.dispatchEvent('model.nostoragefound');
            }

        } else {
            app.common.dispatchEvent('model.noaudiofound');
        }
    }

    /**
     * Error callback for getting directories from ContentManager.
     * Displays message on the console when error occurs
     * while getting directories list from ContentManager.
     *
     * @memberof app.model
     * @private
     * @param {Error} error
     */
    function onGetDirectoriesError(error) {
        console.error('onGetDirectories error: ', error);
    }

    /**
     * Obtains directories list.
     *
     * @memberof app.model
     * @private
     */
    function getDirectoriesList() {
        try {
            tizen.content.getDirectories(onGetDirectoriesSuccess,
                onGetDirectoriesError);
        } catch (error) {
            console.error('getDirectories error:', error);
        }
    }

    /**
     * Sets media keys event listener. Fires events when media keys are pressed,
     * or released.
     * Fires "model.mediakeyplaypressed" event when button play is pressed.
     * Fires "model.mediakeypausepressed" event when button pause is pressed.
     * Fires "model.mediakeystoppressed" event when button stop is pressed.
     * Fires "model.mediakeyffwdpressed" event when button fast forward is
     * pressed.
     * Fires "model.mediakeynextpressed" event when button next is pressed.
     * Fires "model.mediakeyrewindpressed" event when button rewind is pressed.
     * Fires "model.mediakeyprevpressed" event when button previous is pressed.
     * Fires "model.mediakeyplayppressed" event when button play/pause is
     * pressed.
     * Fires "model.mediakeyffwdreleased" event when button fast forward is
     * released.
     * Fires "model.mediakeyrewreleased" event when button rewind is released.
     * Fires "model.mediakeypressed" event if any button is pressed.
     *
     * @memberof app.model
     * @private
     * @fires "model.mediakeyplaypressed"
     * @fires "model.mediakeypausepressed"
     * @fires "model.mediakeystoppressed"
     * @fires "model.mediakeyffwdpressed"
     * @fires "model.mediakeynextpressed"
     * @fires "model.mediakeyrewindpressed"
     * @fires "model.mediakeyprevpressed"
     * @fires "model.mediakeyplayppressed"
     * @fires "model.mediakeyffwdreleased"
     * @fires "model.mediakeyrewreleased"
     * @fires "model.mediakeypressed"
     */
    function setMediaKeyListener() {
        try {
            tizen.mediakey.setMediaKeyEventListener({
                onpressed: function onPressedKey(key) {
                    app.common.dispatchEvent('model.mediakeypressed');

                    switch (key) {
                    case 'MEDIA_PLAY':
                        app.common.dispatchEvent('model.mediakeyplaypressed');
                        break;
                    case 'MEDIA_PAUSE':
                        app.common.dispatchEvent('model.mediakeypausepressed');
                        break;
                    case 'MEDIA_STOP':
                        app.common.dispatchEvent('model.mediakeystoppressed');
                        break;
                    case 'MEDIA_FAST_FORWARD':
                        app.common.dispatchEvent('model.mediakeyffwdpressed');
                        break;
                    case 'MEDIA_NEXT':
                        app.common.dispatchEvent('model.mediakeynextpressed');
                        break;
                    case 'MEDIA_REWIND':
                        app.common.dispatchEvent('model.mediakeyrewindpressed');
                        break;
                    case 'MEDIA_PREVIOUS':
                        app.common.dispatchEvent('model.mediakeyprevpressed');
                        break;
                    case 'MEDIA_PLAY_PAUSE':
                        app.common.dispatchEvent('model.mediakeyplayppressed');
                        break;
                    default:
                        break;
                    }
                },
                onreleased: function onReleasedKey(key) {
                    switch (key) {
                    case  'MEDIA_FAST_FORWARD':
                        app.common.dispatchEvent('model.mediakeyffwdreleased');
                        break;
                    case 'MEDIA_REWIND':
                        app.common.dispatchEvent('model.mediakeyrewreleased');
                        break;
                    default:
                        break;
                    }
                }
            });
        } catch (error) {
            console.error('setMediaKeyListener error :', error);
        }
    }

    /**
     * Binds model events.
     *
     * @memberof app.model
     * @private
     */
    function bindModelEvents() {
        setMediaKeyListener();
    }

    /**
     * Initializes model module.
     *
     * @memberof app.model
     * @public
     */
    function init() {
        getDirectoriesList();
        bindModelEvents();
    }

    app.model = {
        getAudioContentList: getAudioContentList,
        init: init
    };

})(window.app);
