/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* global window, document, tau, console */

/**
 * Application UI module.
 * It's responsible for managing user interface.
 *
 * @module app.ui
 * @namespace app.ui
 * @memberof app
 */

// make sure that "app" namespace is created
window.app = window.app || {};

// strict mode wrapper
(function defineAppUi(app) {
    'use strict';

    /**
     * Stores audio state popup image appear time in milliseconds.
     *
     * @memberof app.ui
     * @private
     * @const {number}
     */
    var AUDI0_STATE_POPUP_APPEAR_TIME = 3000,

        /**
         * Progress bar template.
         *
         * @memberof app.ui
         * @private
         * @const {string}
         */
        PROGRESSBAR_TEMPLATE = '<div id="progress-bar"><img id="state">' +
            '<progress max="100" value ="0" id="progress"></progress>' +
            '<div id="timer">00:00</div>',

        /**
         * Stores TAU SnapListview component.
         *
         * @memberof app.ui
         * @private
         * @type {object}
         */
        snapListWidget = null,

        /**
         * No storage found TAU toast popup.
         *
         * @memberof app.ui
         * @private
         * @type {HTMLElement}
         */
        noStorageFoundPopup = null,

        /**
         * No audio files found TAU toast popup.
         *
         * @memberof app.ui
         * @private
         * @type {HTMLElement}
         */
        noAudioFoundPopup = null,

        /**
         * List container.
         *
         * @memberof app.ui
         * @private
         * @type {HTMLElement}
         */
        audioListContainer = null,

        /**
         * Stores available audio objects.
         *
         * @memberof app.ui
         * @private
         * @type {AudioContent[]}
         */
        audioList = [],

        /**
         * HTML5 audio element.
         *
         * @memberof app.ui
         * @private
         * @type {HTMLElement}
         */
        audio = null,

        /**
         * Container for playing state icon, progress bar and timer.
         *
         * @memberof app.ui
         * @private
         * @type {HTMLElement}
         */
        progressBar = null,

        /**
         * HTML progress element.
         *
         * @memberof app.ui
         * @private
         * @type {HTMLElement}
         */
        progress = null,

        /**
         * HTML img element with audio state icon.
         *
         * @memberof app.ui
         * @private
         * @type {HTMLElement}
         */
        audioStateIcon = null,

        /**
         * Container for audio current time.
         *
         * @memberof app.ui
         * @private
         * @type {HTMLElement}
         */
        timer = null,

        /**
         * Stores active audio list element.
         *
         * @memberof app.ui
         * @private
         * @type {HTMLElement}
         */
        activeAudioListElement = null,

        /**
         * Stores id of previously played audio.
         *
         * @memberof app.ui
         * @private
         * @type {number}
         */
        previouslyPlayedAudioId = 0,

        /**
         * HTML img element with popup image.
         *
         * @memberof app.ui
         * @private
         * @type {HTMLElement}
         */
        audioStatePopupImage = null,

        /**
         * Stores change audio time interval identifier.
         *
         * @memberof app.ui
         * @private
         * @type {number}
         */
        changeAudioTimeInterval = 0,

        /**
         * Stores popup image disappear timeout identifier.
         *
         * @memberof app.ui
         * @private
         * @type {number}
         */
        popupImageDisappearTimeout = 0;

    /**
     * Clears popup image disappearance timeout.
     * It prevents from hiding popup image in the case when user press another
     * button just before previous timeout occurs.
     *
     * @memberof app.ui
     * @private
     */
    function clearPopupImageDisappearTimeout() {
        if (popupImageDisappearTimeout) {
            window.clearTimeout(popupImageDisappearTimeout);
            popupImageDisappearTimeout = 0;
        }
    }

    /**
     * Clears "changeAudioTimeInterval". Changes audio state icon and popup
     * image. It prevents from incorrect operation, when during rewind or
     * fast-forward operation, connection to the BT device is lost.
     *
     * @memberof app.ui
     * @private
     */
    function clearChangeAudioTimeInterval() {
        if (changeAudioTimeInterval) {
            window.clearInterval(changeAudioTimeInterval);
            changeAudioTimeInterval = 0;
            audioStatePopupImage.src = 'images/frame.png';

            if (!audio.paused) {
                audioStateIcon.src = 'images/play.png';
            }
        }
    }

    /**
     * Shows audio list on the page. Initializes list component.
     *
     * @memberof app.ui
     * @private
     */
    function showAudioList() {
        var i = 0,
            listLength = audioList.length,
            listContent = '';

        for (i; i < listLength; i += 1) {
            listContent += '<li data-listid="' + i + '">' +
                '<div class="ui-marquee ui-marquee-gradient">' +
                app.common.escapeHTML(audioList[i].title) +
                '</div></li>';
        }

        audioListContainer.innerHTML = listContent;
        initListComponent();
    }

    /**
     * Initializes TAU SnapListMarqueeStyle helper module.
     * Creates SnapListview widget.
     * Sets active audio list element.
     *
     * @memberof app.ui
     * @private
     */
    function initListComponent() {
        tau.helper.SnapListMarqueeStyle.create(audioListContainer,
            {marqueeDelay: 1000});
        snapListWidget = tau.widget.SnapListview(audioListContainer);
        activeAudioListElement =
            document.querySelector('.ui-snap-listview-selected');
    }

    /**
     * Converts data id of active audio list element to number.
     *
     * @memberof app.ui
     * @private
     * @returns {number}
     */
    function getActiveAudioId() {
        return parseInt(activeAudioListElement.dataset.listid, 10);
    }

    /**
     * Initializes HTML5 audio element on the page.
     *
     * @memberof app.ui
     * @private
     */
    function initAudio() {
        audio = document.createElement('audio');
        document.body.appendChild(audio);
        audio.src = audioList[getActiveAudioId()].contentURI;
    }

    /**
     * Sets source of the audio element.
     * Starts playing audio. Scrolls list to the current audio.
     *
     * @memberof app.ui
     * @private
     * @param {number} audioId
     */
    function playAudio(audioId) {
        audio.src = audioList[audioId].contentURI;
        audio.play();
        snapListWidget.scrollToPosition(audioId);
    }

    /**
     * Creates new progress bar from template, and adds it to the
     * active audio list element (if progress bar element exists, removes it).
     *
     * @memberof app.ui
     * @private
     */
    function showProgressBar() {
        if (progressBar && (previouslyPlayedAudioId !== getActiveAudioId())) {
            progressBar.parentNode.removeChild(progressBar);
            progressBar = null;
        }

        if (!progressBar) {
            activeAudioListElement.insertAdjacentHTML('beforeend',
                PROGRESSBAR_TEMPLATE);
            progressBar = document.getElementById('progress-bar');
            progress = document.getElementById('progress');
            timer = document.getElementById('timer');
            audioStateIcon = document.getElementById('state');
        }
    }

    /**
     * Converts time from number to string for time container.
     *
     * @memberof app.ui
     * @private
     * @param {number} duration
     * @returns {string}
     */
    function getAudioTime(duration) {
        var minutes = 0,
            seconds = 0;

        minutes = Math.floor(duration / 60);
        if (minutes < 10) {
            minutes = '0' + minutes;
        }

        seconds = Math.floor(duration % 60);
        if (seconds < 10) {
            seconds = '0' + seconds;
        }

        return minutes + ':' + seconds;
    }

    /**
     * Hides audio state popup image.
     *
     * @memberof app.ui
     * @private
     */
    function hidePopupImage() {
        audioStatePopupImage.src = 'images/frame.png';
    }

    /**
     * Shows audio state popup image. Sets "hidePopupImage" function timeout.
     *
     * @memberof app.ui
     * @private
     * @param {string} buttonPressed
     */
    function showAudioStatePopupImage(buttonPressed) {
        var popupImageChanged = true;

        switch (buttonPressed) {
        case 'play':
            audioStatePopupImage.src = 'images/play_popup.png';
            break;
        case 'stop':
            audioStatePopupImage.src = 'images/stop_popup.png';
            break;
        case 'pause':
            audioStatePopupImage.src = 'images/pause_popup.png';
            break;
        case 'next':
            audioStatePopupImage.src = 'images/next_popup.png';
            break;
        case 'previous':
            audioStatePopupImage.src = 'images/previous_popup.png';
            break;
        case 'rewind':
            audioStatePopupImage.src = 'images/rewind_popup.png';
            break;
        default:
            console.warn(buttonPressed + ' was not handled');
            popupImageChanged = false;
            break;
        }

        if (popupImageChanged) {
            popupImageDisappearTimeout = window.setTimeout(hidePopupImage,
                AUDI0_STATE_POPUP_APPEAR_TIME);
        }
    }

    /**
     * Handles audio "play" event.
     * Shows progress bar element. Sets audio play state icon.
     * Sets previously played audio id.
     *
     * @memberof app.ui
     * @private
     */
    function onPlay() {
        showProgressBar();
        audioStateIcon.src = 'images/play.png';
        previouslyPlayedAudioId = getActiveAudioId();
    }

    /**
     * Handles audio "ended" event.
     * Shows "stop" state icon.
     *
     * @memberof app.ui
     * @private
     */
    function onEnded() {
        audioStateIcon.src = 'images/stop.png';
    }

    /**
     * Handles audio "onTimeUpdate" event.
     * Sets progress bar value. Shows current audio time.
     *
     * @memberof app.ui
     * @private
     */
    function onTimeUpdate() {
        var progressBarValue = 0;

        if (audio.currentTime > 0) {
            progressBarValue = (parseInt(audio.currentTime, 10) /
                parseInt(audio.duration, 10)) * 100;
        }

        progress.value = progressBarValue;
        timer.innerHTML = getAudioTime(audio.currentTime);
    }

    /**
     * Starts playing or pauses audio.
     *
     * @memberof app.ui
     * @private
     */
    function togglePlayPause() {
        if (audio.paused) {
            audio.play();
            showAudioStatePopupImage('play');
        } else {
            audio.pause();
            showAudioStatePopupImage('pause');
            audioStateIcon.src = 'images/pause.png';
        }
    }

    /**
     * Handles "model.mediakeypressed" event.
     * Launches functions responsible for clearing timeout and interval if any
     * of them exist.
     *
     * @memberof app.ui
     * @private
     */
    function onMediaKeyPressed() {
        clearChangeAudioTimeInterval();
        clearPopupImageDisappearTimeout();
    }

    /**
     * Handles "model.mediakeyplaypausepressed" event.
     * Initializes function which starts playing or pauses audio.
     *
     * @memberof app.ui
     * @private
     */
    function onMediaKeyPlayPausePressed() {
        togglePlayPause();
    }

    /**
     * Handles "model.mediakeyplaypressed" event.
     * Shows "play" popup image. Plays audio.
     *
     * @memberof app.ui
     * @private
     */
    function onMediaKeyPlayPressed() {
        showAudioStatePopupImage('play');

        if (audio.paused) {
            audio.play();
        }
    }

    /**
     * Handles "model.mediakeypausepressed" event.
     * Shows "pause" popup image. If audio is playing pauses it and
     * sets "pause" state icon.
     *
     * @memberof app.ui
     * @private
     */
    function onMediaKeyPausePressed() {
        showAudioStatePopupImage('pause');

        if (!audio.paused) {
            audio.pause();
            audioStateIcon.src = 'images/pause.png';
        }
    }

    /**
     * Handles "model.mediakeystoppressed" event.
     * Shows "stop" popup image.
     * Stops playing audio. Sets current time of the audio to the beginning.
     *
     * @memberof app.ui
     * @private
     */
    function onMediaKeyStopPressed() {
        showAudioStatePopupImage('stop');
        audio.pause();

        if (progressBar) {
            audioStateIcon.src = 'images/stop.png';
            audio.currentTime = 0;
        }
    }

    /**
     * Handles "model.mediakeynextpressed" event.
     * Shows "next" popup image. Forwards list. Sets previously played audio id.
     * Sets active audio list element. Initializes function which starts playing
     * audio.
     *
     * @memberof app.ui
     * @private
     */
    function onMediaKeyNextPressed() {
        showAudioStatePopupImage('next');

        if (getActiveAudioId() < (audioList.length - 1)) {
            if (audioStateIcon) {
                audioStateIcon.src = 'images/next.png';
            }

            previouslyPlayedAudioId = getActiveAudioId();
            activeAudioListElement = activeAudioListElement.nextSibling;
            playAudio(getActiveAudioId());
        }
    }

    /**
     * Increases audio current time.
     * If audio current time is greater than audio duration, removes change
     * audio interval and changes audio popup image.
     *
     * @memberof app.ui
     * @private
     */
    function fastforwardAudio() {
        audio.currentTime += 1;

        if (audio.currentTime >= audio.duration) {
            clearChangeAudioTimeInterval();
            audioStatePopupImage.src = 'images/frame.png';
        }
    }

    /**
     * Handles "model.mediakeyffwdpressed" event.
     * Shows "fast-forward" popup image. Sets interval for "fastForwardAudio"
     * function.
     *
     * @memberof app.ui
     * @private
     */
    function onMediaKeyFfwdPressed() {
        audioStatePopupImage.src = 'images/fastforward_popup.png';

        if (!audio.paused) {
            audioStateIcon.src = 'images/fastforward.png';
            changeAudioTimeInterval = window.setInterval(fastforwardAudio, 300);
        }
    }

    /**
     * Handles "model.mediakeyffwdreleased" event.
     * Removes fast-forward interval if it exists. Sets audio popup image and
     * audio state icon.
     *
     * @memberof app.ui
     * @private
     */
    function onMediaKeyFfwdReleased() {
        clearChangeAudioTimeInterval();
        audioStatePopupImage.src = 'images/frame.png';

        if (!audio.paused) {
            audioStateIcon.src = 'images/play.png';
        }
    }

    /**
     * Handles "model.mediakeypreviouspressed" event.
     * Shows "previous" popup image. Sets previously played audio id. Sets
     * active audio list element. Initializes function which starts playing
     * audio.
     *
     * @memberof app.ui
     * @private
     */
    function onMediaKeyPreviousPressed() {
        showAudioStatePopupImage('previous');

        if (getActiveAudioId() > 0) {
            audioStateIcon.src = 'images/previous.png';
            previouslyPlayedAudioId = getActiveAudioId();
            activeAudioListElement = activeAudioListElement.previousSibling;
            playAudio(getActiveAudioId());
        }
    }

    /**
     * Handles "model.mediakeyrewindpressed" event.
     * Shows "rewind" popup image. Decreases audio current time. Changes audio
     * state icon. Sets interval for "rewindAudio" function.
     *
     * @memberof app.ui
     * @private
     */
    function onMediaKeyRewindPressed() {
        function rewindAudio() {
            audio.currentTime -= 1;
        }

        if (!audio.paused) {
            audio.currentTime -= 1;
            audioStateIcon.src = 'images/rewind.png';
            changeAudioTimeInterval = window.setInterval(rewindAudio, 300);
        }

        audioStatePopupImage.src = 'images/rewind_popup.png';
    }

    /**
     * Handles "model.mediakeyrewreleased" event. Removes rewind interval if it
     * exists. Hides "rewind" popup image and changes audio state icon.
     *
     * @memberof app.ui
     * @private
     */
    function onMediaKeyRewindReleased() {
        clearChangeAudioTimeInterval();
        audioStatePopupImage.src = 'images/frame.png';

        if (!audio.paused) {
            audioStateIcon.src = 'images/play.png';
        }
    }

    /**
     * Handles "model.nostoragefound" event.
     * Shows no storage found popup.
     *
     * @memberof app.ui
     * @private
     */
    function onNoStorageFound() {
        tau.openPopup(noStorageFoundPopup);
    }

    /**
     * Handles "model.noaudiofound" event.
     * Shows no audio files found popup.
     *
     * @memberof app.ui
     * @private
     */
    function onNoAudioFound() {
        tau.openPopup(noAudioFoundPopup);
    }

    /**
     * Registers event listeners which require audio data to be ready.
     *
     * @memberof app.ui
     * @private
     */
    function bindAudioReadyEvents() {
        audio.addEventListener('timeupdate', onTimeUpdate);
        audio.addEventListener('play', onPlay);
        audio.addEventListener('ended', onEnded);
        window.addEventListener('model.mediakeypressed', onMediaKeyPressed);
        window.addEventListener('model.mediakeyplaypressed',
            onMediaKeyPlayPressed);
        window.addEventListener('model.mediakeypausepressed',
            onMediaKeyPausePressed);
        window.addEventListener('model.mediakeystoppressed',
            onMediaKeyStopPressed);
        window.addEventListener('model.mediakeyffwdpressed',
            onMediaKeyFfwdPressed);
        window.addEventListener('model.mediakeynextpressed',
            onMediaKeyNextPressed);
        window.addEventListener('model.mediakeyrewindpressed',
            onMediaKeyRewindPressed);
        window.addEventListener('model.mediakeyprevpressed',
            onMediaKeyPreviousPressed);
        window.addEventListener('model.mediakeyplayppressed',
            onMediaKeyPlayPausePressed);
        window.addEventListener('model.mediakeyffwdreleased',
            onMediaKeyFfwdReleased);
        window.addEventListener('model.mediakeyrewreleased',
            onMediaKeyRewindReleased);
    }

    /**
     * Handles "model.ready" event. Assigns audio content to variable.
     * Initializes list creation function.
     * Initializes audio element.
     * Initializes media key and audio events.
     *
     * @memberof app.ui
     * @private
     */
    function onModelReady() {
        audioList = app.model.getAudioContentList();
        showAudioList();
        initAudio();
        bindAudioReadyEvents();
    }

    /**
     * Registers event listeners which will handle initialization of the user
     * interface.
     *
     * @memberof app.ui
     * @private
     */
    function bindInitEvents() {
        window.addEventListener('model.ready', onModelReady);
        window.addEventListener('model.nostoragefound', onNoStorageFound);
        window.addEventListener('model.noaudiofound', onNoAudioFound);
    }

    /**
     * Initializes UI module.
     *
     * @memberof app.ui
     * @public
     */
    function init() {
        audioListContainer = document.getElementById('audio-list');
        audioStatePopupImage =
            document.getElementById('audio-state-popup-image');
        noAudioFoundPopup = document.getElementById('no-audio-found');
        noStorageFoundPopup = document.getElementById('no-storage-found');
        bindInitEvents();
    }

    app.ui = {
        init: init
    };

})(window.app);
